/*
  Switch.cpp - Library for handling switches
*/

#include <Arduino.h>
#include "ui.h"
#include <GnusbuinoMIDI.h>        // you MUST include the Gnusbuino MIDI library


// hardware oriented descriptoin - put here what you connected where to the gnusbuino board 

    const uint8_t AuxiliaryButton =  2;          // port B2
    const uint8_t OctaveDownButton = 1;          // port B1
    const uint8_t OctaveUpButton =   0;          // port B0
    
    const uint8_t led1 = 7;          // port B7
    const uint8_t led2 = 6;          // port B6
    const uint8_t led3 = 5;          // port B5
    const uint8_t led4 = 4;          // port B4
    const uint8_t led5 = 3;          // port B3  
    
    const uint8_t noteC =   16;          // port C0      
    const uint8_t noteCis = 17;          // port C1
    
    
    
    
    
void Ui::Init() {
  octave = 3;   // set to the middle octave initially
 
  // empty all arrays for the  buttonhandling 
  for (int i = 0; i<27; i++) {
    action_done_[i] = 0;
    release_action_done_[i] = 0;
    press_time_[i] = 0;
    release_time_[i] = 0;
  }
 
  //HARDWARE INIT 
  pinMode(7,OUTPUT);  // led1
  pinMode(6,OUTPUT);  // led2
  pinMode(5,OUTPUT);  // led3
  pinMode(4,OUTPUT);  // led4
  pinMode(3,OUTPUT);  // led5
  pinMode(2,INPUT_PULLUP);      // set to input with pullup for octup octdown and aux button
  pinMode(1,INPUT_PULLUP);  
  pinMode(0,INPUT_PULLUP);
   // pinMode(16,INPUT_PULLUP);  
  // setup the pins for the keyboards keys   from C to H  
  for (int i = 16; i < 27; i++) {
    pinMode(i,INPUT_PULLUP);
  }
  
}

void Ui::Poll() {
  switches_.Debounce(); 
}

void Ui::DoEvents() {
  SwitchOctave();
  PaintOctaveLeds();
  HandleKeyboard();  
  //test();
}

void Ui::SwitchOctave() {

  // OCTAVE UP press handling
  if ( (switches_.pressed(OctaveUpButton)) && (!action_done_[OctaveUpButton]) ) {    // wenn oct up button gedrueckt
   if ( (press_time_[OctaveUpButton] < 10) ) {
     press_time_[OctaveUpButton]++;
   }
   else if ( (switches_.pressed(OctaveUpButton)) && (!action_done_[OctaveUpButton]) && (press_time_[OctaveUpButton] == 10) ) {
     if (octave <5) {
       octave = octave + 1;
     }    
     release_time_[OctaveUpButton]=0;
     action_done_[OctaveUpButton]=1;
     release_action_done_[OctaveUpButton]=0;
   }
  }
  
  // OCTAVE DOWN press handling
  if ( (switches_.pressed(OctaveDownButton)) && (!action_done_[OctaveDownButton]) ) {    // wenn oct up button gedrueckt
   if ( (press_time_[OctaveDownButton] < 10) ) {
     press_time_[OctaveDownButton]++;
   }
   else if ( (switches_.pressed(OctaveDownButton)) && (!action_done_[OctaveDownButton]) && (press_time_[OctaveDownButton] == 10) ) {
     if (octave > 1) {
       octave = octave - 1;
     }    
     release_time_[OctaveDownButton]=0;
     action_done_[OctaveDownButton]=1;
     release_action_done_[OctaveDownButton]=0;
   }
  }

  // octave up release handling 
     if ( (switches_.released(OctaveUpButton)) && (!release_action_done_[OctaveUpButton]) ) {    // wenn oct up button gedrueckt
       if ( (release_time_[OctaveUpButton] < 10) ) {
         release_time_[OctaveUpButton]++;
       }
   else if ( (switches_.released(OctaveUpButton)) && (!release_action_done_[OctaveUpButton]) && (release_time_[OctaveUpButton] == 10) ) {
         // do the magic  
         //MIDI.write(MIDI_NOTEOFF, (i+(octave*12)-4), 0 );  // send a note off when button is really released, 
                                                        // fixed velocity 0, 
                                                        // multiplied by the according octave setting
         press_time_[OctaveUpButton]=0;
         action_done_[OctaveUpButton]=0;
         release_action_done_[OctaveUpButton]=1;
       }
     }
  // octave down release handling    
     if ( (switches_.released(OctaveDownButton)) && (!release_action_done_[OctaveDownButton]) ) {    // wenn oct up button gedrueckt
       if ( (release_time_[OctaveDownButton] < 10) ) {
         release_time_[OctaveDownButton]++;
       }
   else if ( (switches_.released(OctaveDownButton)) && (!release_action_done_[OctaveDownButton]) && (release_time_[OctaveDownButton] == 10) ) {
         // do the magic  
         //MIDI.write(MIDI_NOTEOFF, (i+(octave*12)-4), 0 );  // send a note off when button is really released, 
                                                        // fixed velocity 0, 
                                                        // multiplied by the according octave setting
         press_time_[OctaveDownButton]=0;
         action_done_[OctaveDownButton]=0;
         release_action_done_[OctaveDownButton]=1;
       }
     }
     
}

void Ui::HandleKeyboard() {
//  for (uint8_t i = 16; i <= 27; i++) {

 uint8_t i = 16;    

  // switch pressed handling
  if ( (switches_.pressed(i)) && (!action_done_[i]) ) {    // wenn oct up button gedrueckt
   if ( (press_time_[i] < 10) ) {
     press_time_[i]++;
   }
   else if ( (switches_.pressed(i)) && (!action_done_[i]) && (press_time_[i] == 10) ) {
     // insert magic here
     MIDI.write(MIDI_NOTEON, (i+(octave*12)-4) , 127 );  // send a note on when button is really pressed, 
                                                        // fixed velocity 100, 
                                                        // multiplied by the according octave setting
     release_time_[i]=0;
     action_done_[i]=1;
     release_action_done_[i]=0;
   }
  }
    
  // switch released handling 
     if ( (switches_.released(i)) && (!release_action_done_[i]) ) {    // wenn oct up button gedrueckt
       if ( (release_time_[i] < 10) ) {
         release_time_[i]++;
       }
     else if ( (switches_.released(i)) && (!release_action_done_[i]) && (release_time_[i] == 10) ) {
         // insert magic here
         MIDI.write(MIDI_NOTEOFF, (i+(octave*12)-4), 0 );  // send a note off when button is really released, 
                                                        // fixed velocity 0, 
                                                        // multiplied by the according octave setting
         press_time_[i]=0;
         action_done_[i]=0;
         release_action_done_[i]=1;
       }
     }
   
// }

} 

void Ui::test() {
  MIDI.write(MIDI_NOTEON, 60, 127);
}

void Ui::PaintOctaveLeds() {
 if (octave == 1) {
  digitalWrite(led1,HIGH);
  digitalWrite(led2,LOW);
  digitalWrite(led3,LOW);
  digitalWrite(led4,LOW);
  digitalWrite(led5,LOW); 
  }
  else if (octave == 2) {
  digitalWrite(led1,LOW);
  digitalWrite(led2,HIGH);
  digitalWrite(led3,LOW);
  digitalWrite(led4,LOW);
  digitalWrite(led5,LOW);  
  }
  else if (octave == 3) {
  digitalWrite(led1,LOW);
  digitalWrite(led2,LOW);
  digitalWrite(led3,HIGH);
  digitalWrite(led4,LOW);
  digitalWrite(led5,LOW);  
  }
  else if (octave == 4) {
  digitalWrite(led1,LOW);
  digitalWrite(led2,LOW);
  digitalWrite(led3,LOW);
  digitalWrite(led4,HIGH);
  digitalWrite(led5,LOW);  
  }
  else if (octave == 5) {
  digitalWrite(led1,LOW);
  digitalWrite(led2,LOW);
  digitalWrite(led3,LOW);
  digitalWrite(led4,LOW);
  digitalWrite(led5,HIGH);   
  } 
  else {                   // octave setting to high or low  --> disable the leds
  digitalWrite(led1,LOW);
  digitalWrite(led2,LOW);
  digitalWrite(led3,LOW);
  digitalWrite(led4,LOW);
  digitalWrite(led5,LOW);
  }
 // to high or low - disable the leds
  
}

